/*
* Program to collect and transmit Power Glove sensor data
*/

// Analog Pins
#define pinBend 3
#define pinAccelX 0
#define pinAccelY 1
#define pinAccelZ 2

// Digital Pins
#define pinDPadUp 2
#define pinDPadDown 3
#define pinDPadLeft 4
#define pinDPadRight 5
#define pinB 6
#define pinA 7
#define pinStart 8
#define pinSelect 9
#define pinMuxCtl0 12
#define pinMuxCtl1 13

// Store Bend/Accel values
int bendT = 0;
int bendI = 0;
int bendM = 0;
int bendR = 0;
int accelX = 0;
int accelY = 0;
int accelZ = 0;

// Which finger we're reading
int finger = 0;

// Current state of digital buttons
unsigned int buttons = 0;

boolean connected = false;

void setup()
{
  Serial.begin(9600);
  pinMode(pinDPadUp, INPUT);
  pinMode(pinDPadDown, INPUT);
  pinMode(pinDPadLeft, INPUT);
  pinMode(pinDPadRight, INPUT);
  pinMode(pinB, INPUT);
  pinMode(pinA, INPUT);
  pinMode(pinStart, INPUT);
  pinMode(pinSelect, INPUT);
  pinMode(pinMuxCtl0, OUTPUT);
  pinMode(pinMuxCtl1, OUTPUT);
  
  // Set all digital input pins high (enable pullup resistor)
  // All digital pins will be LOW-on
  digitalWrite(pinDPadUp, HIGH);
  digitalWrite(pinDPadDown, HIGH);
  digitalWrite(pinDPadLeft, HIGH);
  digitalWrite(pinDPadRight, HIGH);
  digitalWrite(pinB, HIGH);
  digitalWrite(pinA, HIGH);
  digitalWrite(pinStart, HIGH);
  digitalWrite(pinSelect, HIGH);
  
  analogReference(DEFAULT);
}

void loop()
{
  while(Serial.available() > 0)
  {
    byte cmd = Serial.read();
    switch(cmd)
    {
      case 'A': // A
        connected = true;
        break;
      case 'D': // D
        connected = false;
        break;
    }
  }
  
  // Early out if we are not yet connected to bluetooth
  if(!connected) return;
  
  // Select and read finger
  switch(finger)
  {
    case 0:
      digitalWrite(pinMuxCtl0, LOW);
      digitalWrite(pinMuxCtl1, LOW);
      delay(10);
      bendT = analogRead(pinBend);
      break;
    case 1:
      digitalWrite(pinMuxCtl0, HIGH);
      digitalWrite(pinMuxCtl1, LOW);
      delay(10);
      bendI = analogRead(pinBend);
      break;
    case 2:
      digitalWrite(pinMuxCtl0, LOW);
      digitalWrite(pinMuxCtl1, HIGH);
      delay(10);
      bendM = analogRead(pinBend);
      break;
    case 3:
      digitalWrite(pinMuxCtl0, HIGH);
      digitalWrite(pinMuxCtl1, HIGH);
      delay(10);
      bendR = analogRead(pinBend);
      break;
  }
  
  // increment finger to read
  finger = (finger + 1) % 4;
  
  // read accelerometer
  accelX = analogRead(pinAccelX);
  accelY = analogRead(pinAccelY);
  accelZ = analogRead(pinAccelZ);
  
  // read digital button state
  buttons = readButtons();
  
  // write the current state of all sensors to the serial line
  writeState();
  delay(10);
}

/*
* Returns an integer bitmask of the current digital button state 
* Konami Code Order -- U-D-L-R-B-A-Start-Select
*/
unsigned int readButtons()
{
  // Negate all readings, as pins are low-enable
  return (!digitalRead(pinDPadUp) << 0) |
    (!digitalRead(pinDPadDown) << 1) | 
    (!digitalRead(pinDPadLeft) << 2) | 
    (!digitalRead(pinDPadRight) << 3) | 
    (!digitalRead(pinB) << 4) | 
    (!digitalRead(pinA) << 5) |
    (!digitalRead(pinStart) << 6) |
    (!digitalRead(pinSelect) << 7);
}

/*
* Writes the current sensor state to the serial out line. Format is:
* Dig. Button State - Thumb - Index - Middle - Ring - Accel X - Accel Y - AccelZ
*/
void writeState()
{
  // Print output -- 
  Serial.print(buttons);
  Serial.print("\t");
  Serial.print(bendT);
  Serial.print("\t");
  Serial.print(bendI);
  Serial.print("\t");
  Serial.print(bendM);
  Serial.print("\t");
  Serial.print(bendR);
  Serial.print("\t");
  Serial.print(accelX);
  Serial.print("\t");
  Serial.print(accelY);
  Serial.print("\t");
  Serial.println(accelZ);
}
